/*==========================================================================
 * Copyright (c) 2011, Soliditet Decision AS, All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Soliditet Decision AS ("Confidential Information"). You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Soliditet Decision.
 *
 * @since 08.apr.2008
=============================================================================*/
package no.soliditet.wsclients;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPMessage;
import javax.xml.ws.Binding;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.handler.Handler;
import javax.xml.ws.handler.MessageContext;
import javax.xml.ws.handler.soap.SOAPHandler;
import javax.xml.ws.handler.soap.SOAPMessageContext;
import java.io.ByteArrayOutputStream;
import java.util.*;

/*
 * This simple SOAPHandler will output the contents of incoming and outgoing messages.
 */
public class SOAPMessageStringHandler implements SOAPHandler<SOAPMessageContext> {

   private static transient final org.apache.commons.logging.Log log = org.apache.commons.logging.LogFactory.getLog(SOAPMessageStringHandler.class);

   // change this to redirect output if desired
   private String inMsg;
   private String outMsg;

   public Set<QName> getHeaders() {
      return null;
   }

   public boolean handleMessage(SOAPMessageContext smc) {
      saveMessage(smc);
      return true;
   }

   public boolean handleFault(SOAPMessageContext smc) {
      saveMessage(smc);
      return true;
   }

   // nothing to clean up
   public void close(MessageContext messageContext) {
   }

   private void saveMessage(SOAPMessageContext smc) {
      SOAPMessage message = smc.getMessage();
      if (isOutBound(smc)) {
         outMsg = getMsgAsString(message);
      }
      else {
         inMsg = getMsgAsString(message);
      }
   }

   private String getMsgAsString(SOAPMessage message) {
      String msg = null;
      try {
         message.setProperty(SOAPMessage.CHARACTER_SET_ENCODING, "iso-8859-1");
         ByteArrayOutputStream baos = new ByteArrayOutputStream();
         message.writeTo(baos);
         msg = baos.toString("iso-8859-1");
      }
      catch (Exception e) {
         log.warn("WS Soap Message convertion failure, message is:\n\"" + msg + "\"");
         log.warn("WS Soap Message convertion failure, call stack: ", e);
      }
      return msg;
   }

   private Boolean isOutBound(SOAPMessageContext smc) {
      return (Boolean)smc.get(MessageContext.MESSAGE_OUTBOUND_PROPERTY);
   }

   public String getInBoundString() {
      return inMsg;
   }

   public String getOutBoundString() {
      return outMsg;
   }

   public static SOAPMessageStringHandler addSoapHandler(BindingProvider bindingProvider) {
      SOAPMessageStringHandler soapHandler;
      Binding binding = bindingProvider.getBinding();
      List<Handler> handlerList = binding.getHandlerChain();
      if (handlerList == null) {
         handlerList = new ArrayList<>();
      }

      soapHandler = new SOAPMessageStringHandler();

      handlerList.add(soapHandler);
      binding.setHandlerChain(handlerList);
      return soapHandler;
   }
}
