package no.soliditet.wsclients.ws3.jaxws;

import no.soliditet.info.util.DmWSClientHelper;
import no.soliditet.info.util.TheStub;
import no.soliditet.util.xml.XmlConstants;
import no.soliditet.util.xml.dom.XmlDOMDocument;
import no.soliditet.util.xml.dom.XmlDOMElement;
import no.soliditet.wsclients.SOAPMessageStringHandler;
import no.soliditet.wsclients.ws3.jaxws.generated.*;
import org.xml.sax.SAXException;

import javax.xml.namespace.QName;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.ws.BindingProvider;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.List;

public class MyJAXWSv3Test {

   private static transient final org.apache.commons.logging.Log log = org.apache.commons.logging.LogFactory.getLog(MyJAXWSv3Test.class);

   private static SOAPMessageStringHandler soapHandler;

   private static URL getUrl(String serverUrl) {
      URL url = null;
      try {
         URL baseUrl = DecisionMakerWebServices3Service.class.getResource(".");
         if (!serverUrl.toLowerCase().endsWith("wsdl")) {
            serverUrl = serverUrl + "/webservices/services/DecisionMakerWebServices3?wsdl";
         }
         url = new URL(baseUrl, serverUrl);
      }
      catch (MalformedURLException e) {
         log.warn("Failed to create URL for the wsdl Location: " + serverUrl, e);
      }
      return url;
   }

   public static DecisionMakerWebServices3 getPort(String serverUrl) throws MalformedURLException {
      DmWSClientHelper.getInstance().setupProxyIfNeeded((TheStub)null, true, false);
      if (log.isDebugEnabled()) {
         DmWSClientHelper.enableJAXWSClientLog(true);
      }

      QName portName = new QName("urn:DecisionMakerWebServices3", "DecisionMakerWebServices3Service");

      DecisionMakerWebServices3Service service = new DecisionMakerWebServices3Service(getUrl(serverUrl), portName);
      DecisionMakerWebServices3 port = service.getDecisionMakerWebServices3();

      // in order to pick up the real request and response XML's, add a customized SOAO handler
      soapHandler = SOAPMessageStringHandler.addSoapHandler((BindingProvider)port);
      return port;
   }

   public static void main(String[] args) {
      try {
         // Construct webservice object using default host name (www.soliditetd.no) or take the name from input argument:
         String hostName = null;
         if ((args != null) && (args.length > 0) && (args[0].trim().length() > 0)) {
            hostName = args[0].trim();
         }

         DecisionMakerWebServices3 decisionWS = getPort(hostName);
         no.soliditet.wsclients.ws3.jaxws.generated.DmWSApplicant applicant = new no.soliditet.wsclients.ws3.jaxws.generated.DmWSApplicant();

         // COMPANY SEARCH:
         applicant.setType("F");
         //applicant.setName("Soliditet Decision");
         applicant.setSsn("937340303");
         applicant.setNationality("no");

         // PERSON SEARCH:
         //applicant.setType("P");
         //applicant.setSsn("");
         //applicant.setFirstName("demo");
         //applicant.setLastName("person");

         // set username and password
         String userName = "system";
         String passWord = "system";

         // Scoring on specific company policies (if any)
         String productName = "Demo Person";

         //decisionWS.score(userName, passWord, null, null, applicant, productName);

         no.soliditet.wsclients.ws3.jaxws.generated.DmWSRequest dmWSRequest = new no.soliditet.wsclients.ws3.jaxws.generated.DmWSRequest();
         dmWSRequest.setUser(userName);
         dmWSRequest.setPassWord(passWord);
         no.soliditet.wsclients.ws3.jaxws.generated.DmWSInformationRequest theRequest= new no.soliditet.wsclients.ws3.jaxws.generated.DmWSInformationRequest();
         DmWSElement addInfoModelName = new DmWSElement();
         addInfoModelName.setName("arg_scoreModel");
         addInfoModelName.setVal(1);
         ArrayOfDmWSElement addInfoArr = new ArrayOfDmWSElement();
         addInfoArr.getItem().add(addInfoModelName);
         theRequest.setAddInfo(addInfoArr);

         //no.soliditet.wsclients.ws3.jaxws.generated.DmWSScoreRequest theRequest = new no.soliditet.wsclients.ws3.jaxws.generated.DmWSScoreRequest();
         ArrayOfDmWSApplicant arrayOfApps = new ArrayOfDmWSApplicant();
         arrayOfApps.getItem().add(applicant);
         theRequest.setApplicants(arrayOfApps);
         //theRequest.setProductName(productName);
         theRequest.setInformationHandler("alpha:products(ho,sc):xml:server");
         ArrayOfDmWSSingleRequest arrayOfSr = new ArrayOfDmWSSingleRequest();
         arrayOfSr.getItem().add(theRequest);
         dmWSRequest.setRequests(arrayOfSr);

         DmWSClientHelper.getInstance().resetProxy((TheStub)null);

//         no.soliditet.wsclients.ws3.jaxws.generated.DmWSResponse rsp = decisionWS.scoreX(dmWSRequest);
         no.soliditet.wsclients.ws3.jaxws.generated.DmWSResponse rsp = decisionWS.getInformationX(dmWSRequest);

         ArrayOfDmWSSingleResponse responses = rsp.getResponses();

         no.soliditet.wsclients.ws3.jaxws.generated.DmWSSingleResponse srspScore;
         srspScore = responses != null && responses.getItem() != null && responses.getItem().size() > 0 ? responses.getItem().get(0) : null;

         handleOneResponse(0, srspScore);

         // Uncomment the below if to get info from Match:it

         //			DmWSInformationRequest irq = new DmWSInformationRequest();
         //			irq.setInformationHandler("match:find:xml:server");
         //			DmWSElement mode = new DmWSElement();
         //			mode.setName("searchMode");
         //			mode.setVal("Exact");
         //			DmWSElement type = new DmWSElement();
         //			type.setName("searchType");
         //			type.setVal("P");
         //			DmWSElement sources = new DmWSElement();
         //			sources.setName("sources");
         //			sources.setVal("101,507");
         //
         //			// Using JAX-RPC:
         //			//irq.setAddInfo(new DmWSElement[]{mode, type, sources});
         //			// Using JAX-WS:
         //			irq.getAddInfo().add(mode);
         //			irq.getAddInfo().add(type);
         //			irq.getAddInfo().add(sources);

         // Uncomment the below if to get raw information (including Decision Score) from provider:

         //			DmWSResponse rsp = decisionWS.getInformation(userName, passWord, null, null, applicant, "alpha:hocrba:xml:server");
         //			List<DmWSSingleResponse> srsps = rsp.getResponses();
         //			int applicantsWritten = 0;
         //			for (DmWSSingleResponse srspInfo : srsps) {
         //				if (srspInfo != null) {
         //					applicantsWritten = handleOneResponse(applicantsWritten, srspInfo);
         //				}
         //			}

         log.debug("Request:" + soapHandler.getOutBoundString());
         log.debug("Response:" + soapHandler.getInBoundString());
      }
      catch (MalformedURLException e) {
         e.printStackTrace();
      }
   }

   private static int handleOneResponse(int applicantsWritten, DmWSSingleResponse singleResponse) {
      if (singleResponse == null) {
         System.out.println("NO RESPONSE!!!");
         return 0;
      }
      else {
         if (applicantsWritten == 0) {
            // output info about the applicant (but only once)
            applicantsWritten = outputApplicantInfo(singleResponse);
         }
         handleOneresponse(singleResponse);
         return applicantsWritten;
      }
   }

   /**
    * Outputs info about the error, scoring or information result
    * 
    * @param singleResponse DmWSSingleResponse object
    */
   private static void handleOneresponse(no.soliditet.wsclients.ws3.jaxws.generated.DmWSSingleResponse singleResponse) {
      if (singleResponse instanceof no.soliditet.wsclients.ws3.jaxws.generated.DmWSError) {
         // we got an error
         handleErrorResponse((no.soliditet.wsclients.ws3.jaxws.generated.DmWSError)singleResponse);
      }
      else if (singleResponse instanceof no.soliditet.wsclients.ws3.jaxws.generated.DmWSInformationResponse) {
         handleInformationResponse((no.soliditet.wsclients.ws3.jaxws.generated.DmWSInformationResponse)singleResponse);
      }
      else if (singleResponse instanceof no.soliditet.wsclients.ws3.jaxws.generated.DmWSScoreResponse) {
         handleScoreResponse((no.soliditet.wsclients.ws3.jaxws.generated.DmWSScoreResponse)singleResponse);
      }
   }

   private static void handleScoreResponse(no.soliditet.wsclients.ws3.jaxws.generated.DmWSScoreResponse scoreResponse) {
      System.out.println("\n\nFrom ScoreResponse object:");
      // If this response only contains old obsolete norwegian scoremodels like 1 or 2, they are of no interest,
      // so we have to check for some we want first. These are models 3, 4, 5 or 6. 4, 5 and 6 never occurs at
      // the same time. model 3 (Payment Index) is delivered simultaneously if subscribed for
      no.soliditet.wsclients.ws3.jaxws.generated.DmWSScoring scoring = null;
      ArrayOfDmWSScoring arrayOfScorings = scoreResponse.getScorings();

      if ((arrayOfScorings != null) && (arrayOfScorings.getItem() != null)) {
         List<DmWSScoring> scorings = arrayOfScorings.getItem();
         for (no.soliditet.wsclients.ws3.jaxws.generated.DmWSScoring dmWSScoring : scorings) {
            if (dmWSScoring != null) {
               String zoneModelId = dmWSScoring.getZoneModelId();
               int modelNo = Integer.parseInt(zoneModelId);
               if ((modelNo == 3) || (modelNo == 4) || (modelNo == 5) || (modelNo == 6)) {
                  scoring = dmWSScoring;
                  // got it, so break
                  break;
               }
            }
         }
      }
      if (scoring == null) {
         // none of interest, ignore it
         return;
      }

      // get zone value:
      int zone = stringToInteger(scoring.getZone());

      // get the decision:
      String result = scoreResponse.getResult();
      String decision = "not set";
      if (result != null) {
         decision = getDecision(result);
      }
      else {
         // if the result is from the raw data from provider (Soliditet Decision) the result is not set.
         // Soliditet Decision delivers the result only as a zone value
         // This means we will have to convert the zone into a decision.
         // We have an array of no.soliditet.wsclients.ws3.jaxws.generated.DmWSZone objects. There is one object per zone, telling what decision that
         // zone results in. In other words, we must loop thru the array, find the correct zone object and take
         // the decision value from there.
         ArrayOfDmWSZone arrayOfZones = scoring.getZones();
         if ((arrayOfZones != null) && (arrayOfZones.getItem() != null)) {
            List<DmWSZone> zones = arrayOfZones.getItem();
            for (no.soliditet.wsclients.ws3.jaxws.generated.DmWSZone wsZone : zones) {
               if (stringToInteger(wsZone.getZone()) == zone) {
                  decision = getDecision(wsZone.getResult());
                  // we got it, no need to look for more
                  break;
               }
            }
         }
      }
      System.out.println("The decision is \"" + decision + "\"");

      // Policyrules may affect the total decision.
      // If a client/applicant is denied, one may want to know why.
      // This is due to either a very low rating zone, or to one or more policy rule(s)
      // that occurred. The applicant may owe a lot of money (payment remarks) and the store (like Tele 2)
      // may not want such customers.

      // The score/rating zone (0 and 1 is deny, 2 is check, 3,4 and 5 is approved)
      //System.out.println("Rating zone is " + scoreResponse.getZone());

      int modelNo = stringToInteger(scoring.getZoneModelId());
      String zoneText = getZoneText(modelNo, zone);
      System.out.println("Kredittsone: " + zone + ": " + zoneText);

      // The following policyrules occured:
      //int policiesFound = 0;
      ArrayOfDmWSPolicyElement policyElements = scoring.getPolicyElements();

      if ((policyElements != null) && (policyElements.getItem() != null)) {
         List<DmWSPolicyElement> pes = policyElements.getItem();
         // Ok, some policyelements occurred, loop thru:
         System.out.println("\n\nThe following policy rules occurred:");
         for (no.soliditet.wsclients.ws3.jaxws.generated.DmWSPolicyElement pe : pes) {
            // print policyname and message
            String peName = pe.getName();
            System.out.print(peName + ": " + getActionMessage(pe));
            // in some cases the customer (like Tele2) wants even more details
            // A policyelement may have an array of no.soliditet.wsclients.ws3.jaxws.generated.DmWSElement objects containing even more info:
            ArrayOfDmWSElement peElements = pe.getElements();
            if ((peElements != null) && (peElements.getItem() != null)) {
               List<DmWSElement> elements = peElements.getItem();
               for (no.soliditet.wsclients.ws3.jaxws.generated.DmWSElement element : elements) {
                  //policiesFound++;
                  // The element has a name and a value,
                  // e.g: "Income: 345000", "Paymentremarks: 4" etc
                  // if only one sub element, don't bother displaying the name once again if the same as parent:
                  String name = element.getName();
                  if (peName.equalsIgnoreCase(name) && (elements.size() == 1)) {
                     System.out.println(", " + element.getVal());
                  }
                  else {
                     System.out.println(name + ": " + element.getVal());
                  }
               }
            }
         }
      }

      // Want to get hold of the totalscore elements? Only returned for actual score, not for raw data:
      // Actually same as for policyelements:
      ArrayOfDmWSTotalScoreElement totalScoreElements = scoring.getTotalScoreElements();

      if ((totalScoreElements != null) && (totalScoreElements.getItem() != null)) {
         List<DmWSTotalScoreElement> tes = totalScoreElements.getItem();
         System.out.println("\n\nThe following total score elements are present:");
         for (no.soliditet.wsclients.ws3.jaxws.generated.DmWSTotalScoreElement te : tes) {
            // print element name and message
            System.out.println("\n" + te.getName() + ": ");
            ArrayOfDmWSAttribute teAttributes = te.getAttributes();
            if ((teAttributes != null) && (teAttributes.getItem() != null)) {
               List<DmWSAttribute> attributes = teAttributes.getItem();
               for (no.soliditet.wsclients.ws3.jaxws.generated.DmWSAttribute attribute : attributes) {
                  System.out.println("\nAttribute name: " + attribute.getName() + " value is \"" + attribute.getText() + "\"");
               }
            }
            // in some cases the customer (like Tele2) wants even more details
            // A total score element may have an array of no.soliditet.wsclients.ws3.jaxws.generated.DmWSElement objects containing even more info:
            ArrayOfDmWSElement teElements = te.getElements();
            if ((teElements != null) && (teElements.getItem() != null)) {
               List<DmWSElement> elements = teElements.getItem();
               for (no.soliditet.wsclients.ws3.jaxws.generated.DmWSElement element : elements) {
                  // The element has a name and a value,
                  System.out.println(element.getName() + ": " + element.getVal());
               }
            }
         }
      }
   }

   /**
    * Finds the text message for the attribute named "action". If none found, null is returned
    * 
    * @param element DmWSElement object
    * @return the text string
    */
   static public String getActionMessage(no.soliditet.wsclients.ws3.jaxws.generated.DmWSElement element) {
      return getAttributeValue(element, "action");
   }

   /**
    * Looks through all attributes for a specific named attribute key
    * 
    * @param element DmWSElement object
    * @param key attribute key
    * @return the text/value associated with the attribute. If none found, null is returned
    */
   static public String getAttributeValue(no.soliditet.wsclients.ws3.jaxws.generated.DmWSElement element, String key) {
      if ((element == null) || (key == null))
         return null;
      ArrayOfDmWSAttribute arrayOfattrs = element.getAttributes();
      if ((arrayOfattrs != null) && (arrayOfattrs.getItem() != null)) {
         List<DmWSAttribute> attrs = arrayOfattrs.getItem();
         for (no.soliditet.wsclients.ws3.jaxws.generated.DmWSAttribute attr : attrs) {
            if (attr != null) {
               if (attr.getName().equals(key)) {
                  // got it, return text
                  return attr.getText();
               }
            }
         }
      }
      return null;
   }

   private static String getZoneText(int modelNo, int zone) {
      String zoneText;
      String modelText;
      if (modelNo == 3) {
         // Decision Payment Index, er en scoringlsning som rangerer foretakene etter sannsynlighet for mislighold.
         modelText = "Decision Payment Index";
         switch (zone) {
            case 1:
               zoneText = "Rd sone";
               break;
            case 2:
               zoneText = "Gul sone";
               break;
            case 3:
               zoneText = "Grnn sone";
               break;
            default:
               zoneText = "Payment Index ikke beregnet";
         }
      }
      else if (modelNo == 5) {
         // Decision Score Company
         modelText = "Decision Score Company";
         switch (zone) {
            case 1:
               zoneText = "Kreditt kun mot sikkerhet";
               break;
            case 2:
               zoneText = "Kreditt mot sikkerhet anbefales";
               break;
            case 3:
               zoneText = "Kredittverdig";
               break;
            case 4:
               zoneText = "Hy kredittverdighet";
               break;
            case 5:
               zoneText = "Hyeste kredittverdighet";
               break;
            default:
               zoneText = "Foretaket er ikke ratet";
               break;
         }
      }
      else {
         // presumeably model 4 (actually ENK companies) or 6, i.e. Decision Score Person
         modelText = "Decision Score " + (modelNo == 6 ? "Person" : "Company");
         switch (zone) {
            case 1:
               zoneText = "Hy risiko";
               break;
            case 2:
               zoneText = "Moderat risiko";
               break;
            case 3:
               zoneText = "Lav risiko";
               break;
            case 4:
               zoneText = "Lav risiko";
               break;
            case 5:
               zoneText = "Lav risiko";
               break;
            default:
               zoneText = "Personen er ikke ratet";
               break;
         }
      }

      System.out.println("\n\nScore model: " + modelText);
      return zoneText;
   }

   private static void handleInformationResponse(no.soliditet.wsclients.ws3.jaxws.generated.DmWSInformationResponse informationResponse) {
      System.out.println("\n\nFrom InformationResponse object:");
      String informationResult = informationResponse.getInformationResult();
      System.out.println("Raw Xml:\n" + informationResult);

      // It could be a scoring in there
      // We could extract some stuff from that using XPath.
      // In this example we've used Dom4J and Xerces

      try {
         XmlDOMDocument doc = new XmlDOMDocument(informationResult, XmlConstants.DEFAULT_ENCODING);
         String szone = doc.getXPathValue("/info/exti/alpha/cr/Rating3/CreditZone/@value");
         String smodelNo = doc.getXPathValue("/info/exti/alpha/cr/Rating3/Model/@value");

         String zoneText = getZoneText(stringToInteger(smodelNo), stringToInteger(szone));
         System.out.println("Kredittsone fra xml: " + szone + ": " + zoneText);

         // Any policycodes?
         List nodeList = doc.getXPathList("/info/exti/alpha/cr/Rating3/Comment/Comment");
         if ((nodeList != null) && (nodeList.size() > 0)) {
            // avoid foreach loop on DOM4J, wn't work
            //noinspection ForLoopReplaceableByForEach
            for (int i = 0; i < nodeList.size(); i++) {
               XmlDOMElement element = XmlDOMElement.getAsElement(nodeList.get(i));
               if (element != null) {
                  System.out.println("Policyname from xml: " + element.getAttributeValue("value"));
               }
            }
         }
      }
      catch (IOException | ParserConfigurationException | SAXException e) {
         e.printStackTrace();
      }
   }

   private static void handleErrorResponse(no.soliditet.wsclients.ws3.jaxws.generated.DmWSError error) {
      System.out.println("Error: " + error.getErrorCode() + ": " + error.getErrorMessage());
      // One may also want to get hold of the error measure
      System.out.println("What to do: " + error.getErrorMeasure());
   }

   private static String getDecision(String result) {
      String decision = "not set";
      switch (result) {
         case "y":
            decision = "approved";
            break;
         case "n":
            decision = "denied";
            break;
         case "c":
            decision = "to be checked";
            break;
      }
      return decision;
   }

   private static int outputApplicantInfo(no.soliditet.wsclients.ws3.jaxws.generated.DmWSSingleResponse singleResponse) {
      int didSome = 0;
      ArrayOfDmWSApplicant arrayOfApps = singleResponse.getApplicants();
      // Unless the score was done with a coapplicant, there is only one applicant, but lets
      // pass thru all:
      if ((arrayOfApps != null) && (arrayOfApps.getItem() != null)) {
         List<DmWSApplicant> apps = arrayOfApps.getItem();
         for (int i = 0; i < apps.size(); i++) {
            DmWSApplicant app = apps.get(i);
            if (app != null) {
               didSome++;
               // more than one applicant returned?
               if (apps.size() > 1) {
                  System.out.println("Applicant no.: " + (i + 1));
               }
               if (app.getType().equals("P")) {
                  System.out.println("Name: " + app.getFirstName() + " " + app.getLastName());
               }
               else {
                  System.out.println("Name: " + app.getName());
               }

               no.soliditet.wsclients.ws3.jaxws.generated.DmWSAddress address = app.getAddress();

               if (address != null) {
                  System.out.println("Address: " + address.getStreet());
                  // ...etc...
               }
               // ...etc...
            }
         }
      }
      return didSome;
   }

   private static int stringToInteger(String str) {
      try {
         return Integer.parseInt(str);
      }
      catch (Exception e) {
         // when unable to convert
         return 0;
      }
   }
}
